"{ Package: 'jn:refactoring_custom' }"

CustomRefactoring subclass:#CustomCodeSelectionToResourceTranslation
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Refactorings'
!

!CustomCodeSelectionToResourceTranslation class methodsFor:'documentation'!

documentation
"
    Wraps variable, literal, string or expression code selection
    with following translation call: resources string:'Some string...'
    Ie.: when 'Some string...' is selected in the code editor then its
    replaced by: resources string:'Some string...'

    [author:]
        Jakub Nesveda <nesvejak@fit.cvut.cz>

"
! !

!CustomCodeSelectionToResourceTranslation class methodsFor:'accessing-presentation'!

description
    "Returns more detailed description of the receiver"

    ^ 'Wrap code selection with translation call - resources string:'

    "Created: / 21-08-2014 / 23:46:18 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

label
    "Returns show label describing the receiver. This label
     is used in UI as menu item/tree item label."

    ^ 'Selection to Resources Translation'

    "Created: / 21-08-2014 / 23:43:03 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeSelectionToResourceTranslation class methodsFor:'queries'!

availableInContext:aCustomContext
    "Returns true if the generator/refactoring is available in given
     context, false otherwise.
     
     Called by the UI to figure out what generators / refactorings
     are available at given point. See class CustomContext for details."
    | codes |

    codes := aCustomContext selectedCodes.

    codes notEmptyOrNil and: [ 
        codes do: [ :selectedCode | 
            (selectedCode selectedSourceCode size > 0) ifTrue: [ 
                ^ true
            ]
        ]
    ].

    ^ false

    "Created: / 21-08-2014 / 23:28:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-10-2014 / 09:30:18 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

availableInPerspective:aCustomPerspective
    "Returns true if the generator/refactoring is available in given
     perspective, false otherwise.
     
     Called by the UI to figure out what generators / refactorings
     to show"

    ^ aCustomPerspective isCodeViewPerspective

    "Created: / 21-08-2014 / 23:28:34 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 14-10-2014 / 10:29:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeSelectionToResourceTranslation methodsFor:'executing'!

buildInContext:aCustomContext
    "Performs a refactoring within given context scope"

    refactoryBuilder 
          replace:'`@expression'
          with:'(resources string: (`@expression))'
          inCodeSelections:aCustomContext selectedCodes

    "Created: / 23-08-2014 / 00:16:58 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 26-08-2014 / 23:11:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomCodeSelectionToResourceTranslation class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

