"{ Package: 'jn:refactoring_custom' }"

TestCase subclass:#CustomRBAbstractClassTests
	instanceVariableNames:'rbClass mock model'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomRBAbstractClassTests class methodsFor:'documentation'!

documentation
"
    Test extensions in RBAbstractClass.

    [author:]
        Jakub Nesveda <nesvejak@fit.cvut.cz>

"
! !

!CustomRBAbstractClassTests methodsFor:'initialization & release'!

setUp

    mock := CustomMock new.
    model := RBNamespace new.
    rbClass := mock mockOf: RBAbstractClass.
    rbClass model: model.

    "Created: / 30-09-2014 / 19:36:05 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 04-10-2014 / 13:46:07 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

tearDown

    mock unmockAll

    "Created: / 30-09-2014 / 19:44:27 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomRBAbstractClassTests methodsFor:'tests'!

test_all_class_var_names_empty
    | expectedClassVars actualClassVars |

    rbClass compileMockMethod: 'allClassVariableNames ^ nil'.  

    expectedClassVars := #().
    actualClassVars := rbClass allClassVarNames.
    
    self assert: expectedClassVars = actualClassVars

    "Created: / 30-09-2014 / 19:40:31 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_all_class_var_names_not_empty
    | expectedClassVars actualClassVars |

    rbClass compileMockMethod: 'allClassVariableNames ^ #(''C1'' ''C2'')'.  

    expectedClassVars := #('C1' 'C2').
    actualClassVars := rbClass allClassVarNames.
    
    self assert: expectedClassVars = actualClassVars

    "Created: / 30-09-2014 / 19:47:13 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_all_superclasses_do_for_object
    | expectedClassNames actualClassNames |

    rbClass := model classNamed: #Object.  

    expectedClassNames := OrderedCollection new.
    actualClassNames := OrderedCollection new.
    rbClass allSuperclassesDo: [ :class |
        actualClassNames add: class name
    ].

    self assert: expectedClassNames = actualClassNames

    "Created: / 04-10-2014 / 13:21:42 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 04-10-2014 / 23:29:49 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_all_superclasses_do_for_object_superclass
    | expectedClassNames actualClassNames |

    rbClass 
        name: #SomeClass;
        superclass: (model classNamed: #Object).  

    expectedClassNames := OrderedCollection new add: #Object; yourself.
    actualClassNames := OrderedCollection new.
    rbClass allSuperclassesDo: [ :class |
        actualClassNames add: class name
    ].

    self assert: expectedClassNames = actualClassNames

    "Created: / 04-10-2014 / 23:28:42 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_all_superclasses_do_with_real_class
    | expectedClassNames actualClassNames realClass |

    "This actually creates realclass"
    realClass := mock mockClassOf: Object.

    rbClass 
        name: #SomeClass;
        superclass: (model classNamed: realClass name asSymbol).  

    expectedClassNames := OrderedCollection new 
        add: realClass name asSymbol;
        add: #Object;
        yourself.

    actualClassNames := OrderedCollection new.
    rbClass allSuperclassesDo: [ :class |
        actualClassNames add: class name
    ].

    self assert: expectedClassNames = actualClassNames

    "Created: / 04-10-2014 / 23:44:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_all_superclasses_do_with_two_superclasses
    | expectedClassNames actualClassNames superclass |

    superclass := RBClass new 
        name: #SomeClass_01;
        superclass: (model classNamed: #Object);
        yourself.

    rbClass 
        name: #SomeClass_02;
        superclass: superclass.  

    expectedClassNames := OrderedCollection new 
        add: #SomeClass_01;
        add: #Object;
        yourself.

    actualClassNames := OrderedCollection new.
    rbClass allSuperclassesDo: [ :class |
        actualClassNames add: class name
    ].

    self assert: expectedClassNames = actualClassNames

    "Created: / 04-10-2014 / 23:38:04 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_compile_method_with_all_filled
    | method compiledMethod change |

    rbClass compileMockMethod: 'isMeta ^ false'.

    method := RBMethod new
        source: 'selector_01 ^ `#literal';
        protocol: 'a test protocol';
        package: 'some_package';
        model: model;
        class: self class;
        method: (self class compiledMethodAt: #test_compile_method_with_all_filled);
        sourceCodeGenerator: (CustomSourceCodeGenerator new
            formatter: CustomNoneSourceCodeFormatter new;
            yourself);
        replace: '`#literal' with: '1';
        selector: #selector_01;
        yourself.

    change := rbClass compileMethod: method.

    self assert: (rbClass includesSelector: #selector_01).

    self assert: 'a test protocol' = (change protocol).
    self assert: 'some_package' = (change package).
    self assert: 'selector_01 ^ 1' = (change source).

    compiledMethod := rbClass compiledMethodAt: #selector_01.

    self assert: 'a test protocol' = (compiledMethod protocol).
    self assert: 'some_package' = (compiledMethod package).
    self assert: 'selector_01 ^ 1' = (compiledMethod source).

    "Created: / 10-10-2014 / 12:17:15 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_compile_method_with_real_method
    | method compiledMethod realMethod change class |

    rbClass compileMockMethod: 'isMeta ^ false'.

    class := mock mockClassOf: Object.
    class new compileMockMethod: 'selector_01 ^ 1'.
    realMethod := class compiledMethodAt: #selector_01.  

    method := RBMethod 
        for: rbClass 
        fromMethod: realMethod
        andSelector: #selector_01.

    change := rbClass compileMethod: method.

    self assert: (rbClass includesSelector: #selector_01).

    self assert: (realMethod category) = (change protocol).
    self assert: (realMethod package) = (change package).
    self assert: 'selector_01 ^ 1' = (change source).

    compiledMethod := rbClass compiledMethodAt: #selector_01.

    self assert: (realMethod category) = (compiledMethod protocol).
    self assert: (realMethod package) = (compiledMethod package).
    self assert: 'selector_01 ^ 1' = (compiledMethod source).

    "Created: / 10-10-2014 / 13:20:44 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_compile_method_with_source
    | method change expectedSource actualSource |

    rbClass compileMockMethod: 'isMeta ^ false'.

    method := RBMethod new
        source: 'selector_01 ^ 1';
        yourself.

    expectedSource := 'selector_01 ^ 1'.
    change := rbClass compileMethod: method.

    self assert: expectedSource = (change source).

    actualSource := (rbClass compiledMethodAt: #selector_01) source.

    self assert: expectedSource = actualSource

    "Created: / 10-10-2014 / 11:56:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_inherits_from_model_class
    | class |

    class := model classNamed: #Object.
    rbClass
        name: #SomeTestClass01;
        superclassName: #Object.

    self assert: (rbClass inheritsFrom: class).

    "Created: / 11-10-2014 / 01:09:01 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (format): / 13-10-2014 / 20:47:51 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_is_subclass_of_model_class
    | class |

    class := model classNamed: #Object.

    rbClass
        name: #SomeTestClass01;
        superclassName: #Object.

    self assert: (rbClass isSubclassOf: class).

    "Created: / 11-10-2014 / 00:57:03 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_is_subclass_of_model_class_not_found
    | class |

    class := RBClass new
        name: #SomeTestClass02;
        yourself.

    rbClass
        name: #SomeTestClass01;
        superclassName: #Object.

    self deny: (rbClass isSubclassOf: class).

    "Created: / 11-10-2014 / 12:22:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_is_subclass_of_model_class_two_level_subclass
    | class |

    class := model classNamed: #Object.

    rbClass
        name: #SomeTestClass01;
        superclass: (RBClass new
            name: #SomeTestClass02;
            model: model;
            superclassName: #Object;
            yourself).

    self assert: (rbClass isSubclassOf: class).

    "Created: / 11-10-2014 / 01:06:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_is_subclass_of_real_class
    | class |

    class := Object.

    rbClass
        name: #SomeTestClass01;
        superclassName: #Object.

    self assert: (rbClass isSubclassOf: class).

    "Created: / 11-10-2014 / 01:00:06 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_is_subclass_of_real_class_two_level_subclass
    | class |

    class := Object.

    rbClass
        name: #SomeTestClass01;
        superclass: (RBClass new
            name: #SomeTestClass02;
            model: model;
            superclassName: #Object;
            yourself).

    self assert: (rbClass isSubclassOf: class).

    "Created: / 11-10-2014 / 01:05:12 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_method_dictionary_add_and_remove_methods
    | expectedMethodDictionary actualMethodDictionary expectedMethodDictionaryKeys actualMethodDictionaryKeys |

    rbClass compileMockMethod: 'isMeta ^ false'.

    expectedMethodDictionary := MethodDictionary 
        withKeys: (Array with: #selector_01 with: #selector_03) 
        andValues: (Array with: Method new with: Method new).

    rbClass
        compile: 'selector_01 ^ 1';
        compile: 'selector_02 ^ 2';
        compile: 'selector_03 ^ 3';
        removeMethod: #selector_02.

    actualMethodDictionary := rbClass methodDictionary.
    expectedMethodDictionaryKeys := expectedMethodDictionary keys asSortedCollection.
    actualMethodDictionaryKeys := actualMethodDictionary keys asSortedCollection.
    
    self assert: expectedMethodDictionaryKeys = actualMethodDictionaryKeys

    "Created: / 30-09-2014 / 20:02:39 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 30-09-2014 / 21:54:36 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_method_dictionary_add_and_remove_methods_with_real_class
    | expectedMethodDictionary actualMethodDictionary expectedMethodDictionaryKeys actualMethodDictionaryKeys realSelector |

    realSelector := #test_method_dictionary_add_and_remove_methods_with_real_class.

    rbClass compileMockMethod: 'isMeta ^ false'.
    rbClass realClass: self class.

    expectedMethodDictionary := self class methodDictionary 
        at: #selector_01 putOrAppend: Method new.
    expectedMethodDictionary := expectedMethodDictionary 
        at: #selector_03 putOrAppend: Method new.
    expectedMethodDictionary := expectedMethodDictionary removeKeyAndCompress: realSelector.

    rbClass
        compile: 'selector_01 ^ 1';
        compile: 'selector_02 ^ 2';
        compile: 'selector_03 ^ 3';
        removeMethod: #selector_02;
        removeMethod: #test_method_dictionary_add_and_remove_methods_with_real_class.

    actualMethodDictionary := rbClass methodDictionary.
    expectedMethodDictionaryKeys := expectedMethodDictionary keys asSortedCollection.
    actualMethodDictionaryKeys := actualMethodDictionary keys asSortedCollection.
    
    self assert: expectedMethodDictionaryKeys = actualMethodDictionaryKeys

    "Created: / 30-09-2014 / 21:55:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 02-10-2014 / 21:11:46 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_method_dictionary_empty
    | expectedMethodDictionary actualMethodDictionary |

    expectedMethodDictionary := MethodDictionary new.
    actualMethodDictionary := rbClass methodDictionary.
    
    self assert: (expectedMethodDictionary keys) = (actualMethodDictionary keys)

    "Created: / 30-09-2014 / 19:56:32 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_name_without_prefix_non_prefixed_class
    | expectedClassName actualClassName |

    rbClass name: #SomeClass.
    expectedClassName := #SomeClass.
    actualClassName := rbClass nameWithoutPrefix.  

    self assert: expectedClassName = actualClassName

    "Created: / 05-10-2014 / 00:13:21 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_name_without_prefix_prefixed_class
    | expectedClassName actualClassName |

    rbClass name: #Prefix::SomeClass.
    expectedClassName := #SomeClass.
    actualClassName := rbClass nameWithoutPrefix.  

    self assert: expectedClassName = actualClassName

    "Created: / 05-10-2014 / 00:13:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_package_custom
    | expectedPackage actualPackage |

    expectedPackage := 'some_package'.
    rbClass package: 'some_package'.

    actualPackage := rbClass package.

    self assert: expectedPackage = actualPackage

    "Created: / 09-10-2014 / 23:40:08 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_package_empty
    | expectedPackage actualPackage |

    expectedPackage := nil.
    actualPackage := rbClass package.

    self assert: expectedPackage = actualPackage

    "Created: / 09-10-2014 / 23:39:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_package_from_real_class
    | expectedPackage actualPackage |

    expectedPackage := self class package.

    self assert: expectedPackage size > 3.

    rbClass realClass: self class.

    actualPackage := rbClass package.

    self assert: expectedPackage = actualPackage

    "Created: / 09-10-2014 / 23:37:01 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_superclass_name
    | expectedClassName actualClassName |

    rbClass
        name: #SomeClass;
        superclassName: (self class name).

    expectedClassName := self class name.
    actualClassName := rbClass superclass name.

    self assert: expectedClassName = actualClassName

    "Created: / 05-10-2014 / 00:16:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_with_all_superclasses_do_for_one_superclass
    | expectedClassNames actualClassNames |

    rbClass
        name: #SomeClass;
        superclassName: #Object.

    expectedClassNames := OrderedCollection new
        add: #SomeClass;
        add: #Object;
        yourself.

    actualClassNames := OrderedCollection new.

    rbClass withAllSuperclassesDo: [ :class |
        actualClassNames add: class name.
    ].

    self assert: expectedClassNames = actualClassNames

    "Created: / 05-10-2014 / 00:22:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

