"{ Package: 'jn:refactoring_custom' }"

CustomCodeGeneratorOrRefactoringTestCase subclass:#CustomRefactoryBuilderTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomRefactoryBuilderTests methodsFor:'accessing'!

generatorOrRefactoring

    ^ nil

    "Created: / 24-08-2014 / 21:35:05 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomRefactoryBuilderTests methodsFor:'tests'!

test_execute_replace_with_in_code_selection_01
    
    |originalSource expectedSource actualSource codeSelection|

    originalSource := 'selector
    self information: ''Translate this''.
    ^ self.'.

    expectedSource := 'selector
    self information:(resources string:''Translate this'').
    ^ self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: originalSource;
        selectedInterval: (32 to: 47);
        selectedMethod: (self class compiledMethodAt: #generatorOrRefactoring). "/ just some method

    refactoryBuilder executeReplace: '`@code' 
        with: '(resources string: `@code)'
        inCodeSelection: codeSelection.

    actualSource := refactoryBuilder changes last source.

    self assertSource: expectedSource sameAs: actualSource.

    "Created: / 25-08-2014 / 21:53:39 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 26-08-2014 / 22:35:26 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_execute_replace_with_in_code_selection_02
    
    |originalSource expectedSource actualSource codeSelection|

    originalSource := 'selector
    self information: ''Translate this''.
    ^ self.'.

    expectedSource := 'selector
    resources string:(self information:''Translate this'').
    ^ self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: originalSource;
        selectedInterval: (14 to: 47);
        selectedMethod: (self class compiledMethodAt: #generatorOrRefactoring). "/ just some method

    refactoryBuilder executeReplace: '`@code' 
        with: '(resources string: `@code)'
        inCodeSelection: codeSelection.

    actualSource := refactoryBuilder changes last source.

    self assertSource: expectedSource sameAs: actualSource.

    "Created: / 26-08-2014 / 22:37:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_execute_replace_with_in_code_selection_03

    |originalSource expectedSource actualSource codeSelection|

    originalSource := 'selector
    self information: ''Translate this''.
    ^ self.'.

    expectedSource := 'selector
    self information:''Translate this''.
    ^ resources string:self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: originalSource;
        selectedInterval: (55 to: 59);
        selectedMethod: (self class compiledMethodAt: #generatorOrRefactoring). "/ just some method

    refactoryBuilder executeReplace: '`@code' 
        with: '(resources string: `@code)'
        inCodeSelection: codeSelection.

    actualSource := refactoryBuilder changes last source.

    self assertSource: expectedSource sameAs: actualSource.

    "Created: / 26-08-2014 / 22:41:35 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_execute_replace_with_in_code_selection_04

    |originalSource expectedSource actualSource codeSelection|

    originalSource := 'selector
    | temp |

    temp := ''Translate this''.

    self information: temp.
    ^ self.'.

    expectedSource := 'selector
    |temp|

    temp := ''Translate this''.
    self information:(resources string:temp).
    ^ self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: originalSource;
        selectedInterval: (77 to: 81);
        selectedMethod: (self class compiledMethodAt: #generatorOrRefactoring). "/ just some method

    refactoryBuilder executeReplace: '`@code' 
        with: '(resources string: `@code)'
        inCodeSelection: codeSelection.

    actualSource := refactoryBuilder changes last source.

    self assertSource: expectedSource sameAs: actualSource.

    "Created: / 26-08-2014 / 22:52:47 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_search_in_code_selections_with_result_do_01

    |source codeSelection resultsFoundCount |

    source := 'selector
    self information: ''Translate this''.
    ^ self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: source;
        selectedInterval: (32 to: 47).

    resultsFoundCount := 0.

    refactoryBuilder search: '`@code' 
        inCodeSelections: (Array with: codeSelection) 
        withResultDo: [ :codeSelection |
            resultsFoundCount := resultsFoundCount + 1
        ].

    self assert: resultsFoundCount = 1.

    "Created: / 24-08-2014 / 21:10:01 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 25-08-2014 / 10:07:45 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_search_in_code_selections_with_result_do_02

    |source codeSelection resultsFoundCount |

    source := 'selector
    self information: ''Translate this''.
    ^ self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: source;
        selectedInterval: (32 to: 47).

    resultsFoundCount := 0.

    refactoryBuilder search: '`#literal' 
        inCodeSelections: (Array with: codeSelection) 
        withResultDo: [ :codeSelection |
            resultsFoundCount := resultsFoundCount + 1
        ].

    self assert: resultsFoundCount = 1.

    "Created: / 25-08-2014 / 17:00:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_search_in_code_selections_with_result_do_03

    |source codeSelection resultsFoundCount |

    source := 'selector
    self information: ''Translate this''.
    ^ self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: source;
        selectedInterval: (1 to: source size).

    resultsFoundCount := 0.

    refactoryBuilder search: '`#literal' 
        inCodeSelections: (Array with: codeSelection) 
        withResultDo: [ :codeSelection |
            resultsFoundCount := resultsFoundCount + 1
        ].

    self assert: resultsFoundCount = 1.

    "Created: / 25-08-2014 / 17:01:37 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_search_in_code_selections_with_result_do_04

    |source codeSelection resultsFoundCount |

    source := 'selector
    self information: ''Translate this''.
    ^ self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: source;
        selectedInterval: (14 to: 51).

    resultsFoundCount := 0.

    refactoryBuilder search: '`@code' 
        inCodeSelections: (Array with: codeSelection) 
        withResultDo: [ :codeSelection |
            resultsFoundCount := resultsFoundCount + 1
        ].

    self assert: resultsFoundCount = 1.

    "Created: / 25-08-2014 / 17:02:43 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_search_in_code_selections_with_result_do_05

    |source codeSelection resultsFoundCount |

    source := 'selector
    self information: ''Translate this''.
    ^ self.'.

    codeSelection := CustomSourceCodeSelection new.
    codeSelection
        currentSourceCode: source;
        selectedInterval: (30 to: 38).

    resultsFoundCount := 0.

    self should: [ 
        refactoryBuilder search: '`@code' 
            inCodeSelections: (Array with: codeSelection) 
            withResultDo: [ :codeSelection |
                resultsFoundCount := resultsFoundCount + 1
            ].
    ] raise: Error.

    "Created: / 25-08-2014 / 17:03:58 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

