"{ Package: 'jn:refactoring_custom' }"

CustomCodeGeneratorOrRefactoringTestCase subclass:#CustomSubclassResponsibilityCodeGeneratorTests
	instanceVariableNames:'codeGenerator mockSuperClass mockClass builder'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomSubclassResponsibilityCodeGeneratorTests methodsFor:'accessing'!

generatorOrRefactoring

    ^ CustomSubclassResponsibilityCodeGenerator new

    "Created: / 29-08-2014 / 21:41:14 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGeneratorTests methodsFor:'initialization & release'!

setUp

    super setUp.
    mockSuperClass := model createClassImmediate: 'MockSuperClassForTestCase' superClassName: 'Object'.
    mockClass := model createClassImmediate: 'MockClassForTestCase' superClassName: (mockSuperClass new className).

    "Created: / 14-04-2014 / 17:16:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 29-08-2014 / 21:39:57 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGeneratorTests methodsFor:'tests'!

test_none_subclass_responsibility

    context addSelectedClass: mockClass.

    model createMethodImmediate: mockSuperClass protocol: 'instance-protocol' source: 'instanceMethod: aParam
    ^ self'.

    self assert: (mockClass includesSelector: #instanceMethod:) not. 
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    generatorOrRefactoring executeInContext: context.

    self assert: (mockClass includesSelector: #instanceMethod:) not.
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    "Created: / 13-05-2014 / 21:49:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 29-08-2014 / 21:40:53 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_subclass_responsibility_class_method
    | expectedSource |

    context addSelectedClass: mockClass.

    model createMethodImmediate: mockSuperClass class protocol: 'class-protocol' source: 'classMethod: aParam
    self subclassResponsibility'.

    self assert: (mockClass includesSelector: #classMethod:) not.
    self assert: (mockClass class includesSelector: #classMethod:) not.

    generatorOrRefactoring executeInContext: context.

    self assert: (mockClass includesSelector: #classMethod:) not.
    self assert: (mockClass class includesSelector: #classMethod:). 

    expectedSource := 'classMethod:aParam
    self shouldImplement'.

    self assertMethodSource: expectedSource atSelector: #classMethod: forClass: mockClass class.

    "Created: / 15-04-2014 / 22:05:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (format): / 29-08-2014 / 22:15:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_subclass_responsibility_instance_method

    |expectedSource|

    context addSelectedClass: mockClass.

    model createMethodImmediate: mockSuperClass protocol: 'instance-protocol' source: 'instanceMethod: aParam
    self subclassResponsibility'.

    self assert: (mockClass includesSelector: #instanceMethod:) not. 
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    generatorOrRefactoring executeInContext: context.

    self assert: (mockClass includesSelector: #instanceMethod:).
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    expectedSource := 'instanceMethod:aParam
    self shouldImplement'.

    self assertMethodSource: expectedSource atSelector: #instanceMethod: forClass: mockClass.

    "Created: / 14-04-2014 / 18:10:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 29-08-2014 / 22:21:00 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

