"{ Package: 'jn:refactoring_custom' }"

CustomCodeGenerator subclass:#CustomTestCaseMethodCodeGenerator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Generators'
!


!CustomTestCaseMethodCodeGenerator class methodsFor:'accessing-presentation'!

description
    "Returns more detailed description of the receiver"

    ^ 'Creates test method for TestCase from method selection'

    "Created: / 24-08-2014 / 16:16:09 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

group
    "Returns a collection strings describing a group to which
     receiver belongs. A groups may be nested hence the array of
     strings. For example for subgroup 'Accessors' in group 'Generators'
     this method should return #('Generators' 'Accessors')."

    "/ By default return an empty array which means the item will appear
    "/ in top-level group.
    ^ #('Testing')

    "Created: / 05-08-2014 / 14:14:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

label
    "Returns show label describing the receiver. This label
     is used in UI as menu item/tree item label."

    ^ 'Test method'

    "Created: / 24-08-2014 / 16:14:16 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomTestCaseMethodCodeGenerator class methodsFor:'queries'!

availableInContext:aCustomContext
    "Returns true if the generator/refactoring is available in given
     context, false otherwise.
     
     Called by the UI to figure out what generators / refactorings
     are available at given point. See class CustomContext for details."

    ^ aCustomContext selectedMethods notEmptyOrNil

    "Created: / 24-08-2014 / 16:18:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

availableInPerspective:aCustomPerspective
    "Returns true if the generator/refactoring is available in given
     perspective, false otherwise.
     
     Called by the UI to figure out what generators / refactorings
     to show"

    ^ aCustomPerspective isMethodPerspective

    "Created: / 24-08-2014 / 16:16:49 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomTestCaseMethodCodeGenerator methodsFor:'executing'!

buildInContext: aCustomContext

    aCustomContext selectedMethods do:[ :method | 
        | className testClass |

        className := method mclass theNonMetaclass name.

        (className endsWith: 'Tests') ifFalse: [ 
            | testClass |

            className := className, 'Tests'.
            testClass := Smalltalk classNamed: className.
            testClass notNil ifTrue: [ 
                | selector testSelector |

                selector := 'test_', (method selector asString copyReplaceAll: ':' with: '_').
                testSelector := ''.
                selector do:[ :character | 
                    character isUppercase ifTrue: [ 
                        testSelector := testSelector, '_'.
                    ].
                    testSelector := testSelector, character asString toLowerCase
                ].

                (testClass includesSelector: testSelector asSymbol) ifFalse: [ 
                    model createMethod
                        package: method package;
                        class: testClass;
                        protocol: 'tests';
                        source: (testSelector, '                            
                            "/ something ', (method methodDefinitionTemplate asString), '.    
                            self assert: 1 = 2.
                        ');
                        compile
                ].
            ].
        ].
    ].

    "Created: / 24-08-2014 / 16:24:04 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 10-10-2014 / 23:58:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomTestCaseMethodCodeGenerator class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

